package de.intevation.flys.artifacts.model;

import java.util.List;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;

import org.hibernate.Session;

import org.hibernate.SQLQuery;
import org.hibernate.type.StandardBasicTypes;

import de.intevation.flys.artifacts.model.WQKms;

import de.intevation.flys.artifacts.cache.CacheFactory;

import de.intevation.flys.backend.SessionHolder;

/**
 * Factory to access ready-made WQKms for other (than computed) 'kinds' of 
 * WST-data.
 */
public class WQKmsFactory
{
    private static Logger log = Logger.getLogger(WQKmsFactory.class);

    /** Query to get km and wqs for wst_id and column_pos. */
    public static final String SQL_SELECT_WQS =
        "SELECT position, w, q FROM wst_value_table " +
        "WHERE wst_id = :wst_id AND column_pos = :column_pos";

    /** Query to get name for wst_id and column_pos. */
    public static final String SQL_SELECT_NAME =
        "SELECT name " +
        "FROM wst_columns "+
        "WHERE wst_id = :wst_id AND position = :column_pos";


    /** Hidden constructor, use static methods instead. */
    private WQKmsFactory() {
        ;
    }


    /**
     * Get WKms for given column and wst_id, caring about the cache.
     */
    public static WQKms getWQKms(int column, int wst_id) {
        log.debug("WQKmsFactory.getWQKms");
        Cache cache = CacheFactory.getCache(StaticWQKmsCacheKey.CACHE_NAME);

        StaticWQKmsCacheKey cacheKey;

        if (cache != null) {
            cacheKey = new StaticWQKmsCacheKey(wst_id, column);
            Element element = cache.get(cacheKey);
            if (element != null) {
                log.debug("Got static wst values from cache");
                return (WQKms)element.getValue();
            }
        }
        else {
            cacheKey = null;
        }

        WQKms values = getWQKmsUncached(column, wst_id);

        if (values != null && cacheKey != null) {
            log.debug("Store static wst values in cache.");
            Element element = new Element(cacheKey, values);
            cache.put(element);
        }
        return values;
    }


    /**
     * Get WQKms from db.
     * @param column the position columns value
     * @param wst_id database id of the wst
     * @return respective WQKms.
     */
    public static WQKms getWQKmsUncached(int column, int wst_id) {

        if (log.isDebugEnabled()) {
            log.debug("WQKmsFactory.getWQKmsUncached, column "
                + column + ", wst_id " + wst_id);
        }

        WQKms wqkms = new WQKms(WKmsFactory.getWKmsName(column, wst_id));

        Session session = SessionHolder.HOLDER.get();
        SQLQuery sqlQuery = session.createSQLQuery(SQL_SELECT_WQS)
            .addScalar("position", StandardBasicTypes.DOUBLE)
            .addScalar("w",  StandardBasicTypes.DOUBLE)
            .addScalar("q",  StandardBasicTypes.DOUBLE);
        sqlQuery.setInteger("wst_id",     wst_id);
        sqlQuery.setInteger("column_pos", column);

        List<Object []> results = sqlQuery.list();

        double kms [] = new double[results.size()];
        double ws  [] = new double[results.size()];
        double qs  [] = new double[results.size()];

        int lastColumn = Integer.MAX_VALUE;

        for (int i = 0, N = results.size(); i < N; i++) {
            Object[] row = results.get(i);
            // add(w, q, km)
            wqkms.add((Double) row[1], (Double) row[2], (Double) row[0]);
        }

        return wqkms;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
